Imports IntelliBackup.DataMedium

Public Class frmIntelliBackup
    'Eine Liste mit Dateien und das DataBinding
    Private m_FileDataList As List(Of DataMediumFile)
    Private m_BS As BindingSource
    'Die berechnete Aufteilung auf die Medien
    Private m_MediumList As List(Of DataMedium)

    Private Sub frmIntelliBackup_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Load
        'Initalisiert alles
        m_FileDataList = New List(Of DataMediumFile)
        m_MediumList = New List(Of DataMedium)
        m_BS = New BindingSource(m_FileDataList, "")
        FileList.DataSource = m_BS
    End Sub

    'Fgt eine Datei der Liste hinzu
    Private Sub cmdAdd_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdAdd.Click
        'Text in Zahl umwandeln
        Dim size As Long = -1
        Long.TryParse(txtSize.Text, size)

        'Gltigkeit der Eingaben prfen
        If DataMediumFile.ValidName(txtName.Text) And _
          DataMediumFile.ValidSize(size) Then
            'Objekt erstellen und hinzufgen
            m_BS.Add(New DataMediumFile(txtName.Text, size))

            'Fr die komfortable Eingabe
            txtName.Text = "Datei" & CStr(m_BS.Count + 1)
            txtSize.Text = ""
            txtSize.Focus()
        Else
            'Benachrichtigung
            MsgBox("Sie haben keine gltigen Daten eingegeben!", _
                MsgBoxStyle.Information)
        End If
    End Sub

    Private Sub txtSize_KeyUp(ByVal sender As Object, ByVal e As System.Windows.Forms.KeyEventArgs) Handles txtSize.KeyUp
        'Fr die komfortable Eingabe
        If e.KeyCode = Keys.Enter Then cmdAdd_Click(Me, Nothing)
    End Sub

    Private Sub cmdDelete_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdDelete.Click
        If FileList.SelectedIndex <> -1 Then
            'Nachfragen
            Dim res As MsgBoxResult = MsgBox("Wollen Sie den ausgewhlten Eintrag lschen?", MsgBoxStyle.YesNo)
            If res = MsgBoxResult.Yes Then
                m_BS.RemoveAt(FileList.SelectedIndex)
            End If
        End If
    End Sub

    Private Sub cmdDistribute_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdDistribute.Click
        'Kopie zum Bearbeiten der Daten
        Dim tmp(m_BS.Count - 1) As DataMediumFile
        m_BS.CopyTo(tmp, 0)

        'Die Funktion mit der eigentlichen Logik aufrufen
        m_MediumList = Distribute(New List(Of DataMediumFile)(tmp))

        'Alle alten Eintrge lschen
        TreeViewCDs.Nodes(0).Nodes.Clear()

        'Ergebnis ausgeben
        For i As Integer = 0 To m_MediumList.Count - 1
            'Eintrag fr eine CD erstellen
            Dim newNode As New TreeNode("CD" & CStr(i + 1))
            newNode.Tag = m_MediumList(i)
            'Die zur CD gehrigen Dateien hinzufgen
            For j As Integer = 0 To m_MediumList(i).Files.Count - 1
                newNode.Nodes.Add(m_MediumList(i).Files(j).ToString)
            Next
            'Eintrag hinzufgen
            TreeViewCDs.Nodes(0).Nodes.Add(newNode)
        Next

        'Anzeigeoptionen
        TreeViewCDs.Nodes(0).ExpandAll()
        'Dieser Trick ist notwendig damit das AfterSelect Ereignis sicher auftritt
        TreeViewCDs.SelectedNode = Nothing
        TreeViewCDs.SelectedNode = TreeViewCDs.Nodes(0)
        TreeViewCDs.Select()
    End Sub

    Private Sub TreeViewCDs_AfterSelect(ByVal sender As Object, ByVal e As System.Windows.Forms.TreeViewEventArgs) Handles TreeViewCDs.AfterSelect
        'Schauen, ob Eintrag ausgewhlt ist
        If TreeViewCDs.SelectedNode Is Nothing Then Exit Sub

        If TreeViewCDs.SelectedNode.Text.StartsWith("CD") Then
            'Nhere Informationen zur ausgewhlten CD ausgeben
            Dim item As DataMedium = CType(TreeViewCDs.SelectedNode.Tag, DataMedium)
            lblInfo.Text = "Unbentzter CD-Speicherplatz: " & item.SizeLeft.ToString("D") & " MB" & _
                vbNewLine & "Datei-Anzahl: " & item.Files.Count
        Else
            'Sonst allgemeine Informationen ausgeben
            Dim totalSize As Long
            For Each item As DataMedium In m_MediumList
                totalSize += DataMedium.c_MaxSize - item.SizeLeft
            Next
            lblInfo.Text = "Backup-Gre: " & totalSize.ToString("D") & " MB" & _
                vbNewLine & "CD-Anzahl: " & m_MediumList.Count
        End If
    End Sub

    '********************************************************************************************
    ' Diese Funktion verteilt Dateien optimal auf mehrere CDs
    ' nhere Beschreibung siehe HTML-Datei im Projekt
    '********************************************************************************************
    Public Function Distribute(ByVal FileList As List(Of DataMediumFile)) As List(Of DataMedium)
        Dim CDList As New List(Of DataMedium)
        'Sortieren der Dateien
        FileList.Sort()

        'So lange arbeiten, bis alle Ausgangsdateien verarbeitet wurden
        Do Until FileList.Count = 0
            'Eine CD erstellen
            Dim CDitem As New DataMedium
            'Diese Variable speichert, ob der CD eine Datei hinzugefgt wurde
            Dim action As Boolean
            'Diese Schleife luft so lange, bis der CD keine Datei mehr hinzugefgt werden kann
            Do
                'Hilfsvariable zurcksetzen
                action = False
                'Alle Dateien von der grten beginnend durchlaufen
                For i As Integer = FileList.Count - 1 To 0 Step -1
                    'Prfen, ob die Datei auf der CD noch Platz hat
                    If CDitem.SizeLeft >= FileList(i).Size Then
                        'Datei der CD hinzufgen
                        CDitem.AddFile(FileList(i))
                        'Aus der ursprnglichen Liste lschen
                        FileList.RemoveAt(i)
                        'Es wurde eine Datei hinzugefgt
                        action = True
                        'Schleife verlassen
                        Exit For
                    End If
                Next
            Loop Until Not (action)
            'Die CD der Liste hinzufgen
            CDList.Add(CDitem)
        Loop
        'Die CD-Liste zurckgeben
        Return CDList
    End Function
    '********************************************************************************************
End Class
